// SPDX-License-Identifier: Unlicense
pragma solidity ^0.8.24;

import "./WorkContract.sol";
import "./Rollup.sol";

abstract contract RollableWorkContract is WorkContract {
    Rollup immutable rollupContract;

    constructor(
        DAO dao,
        Proposals proposalsContract,
        uint price,
        Rollup rollupContract_
    ) WorkContract(dao, proposalsContract, price) {
        rollupContract = rollupContract_;
    }

    /// Accept work approval/disapproval from customer
    function submitWorkApproval(
        uint requestIndex,
        bool approval
    ) external override {
        WorkRequest storage request = requests[requestIndex];
        require(
            request.status == WorkStatus.EvidenceSubmitted,
            "Status must be EvidenceSubmitted"
        );
        AvailabilityStake storage stake = stakes[request.stakeIndex];
        request.status = WorkStatus.ApprovalSubmitted;
        request.approval = approval;
        emit WorkApprovalSubmitted(requestIndex, approval);

        // Make work evidence post
        Author[] memory authors = new Author[](1);
        authors[0] = Author(1000000, stake.worker);

        // TODO: send post and fee to rollup contract
        dao.forwardAllowance(
            stake.worker,
            address(rollupContract),
            stake.amount
        );
        payable(address(rollupContract)).transfer(request.fee);
        rollupContract.addItem(
            stake.worker,
            stake.amount,
            request.fee,
            request.evidenceContentId
        );

        // dao.addPost(authors, request.evidenceContentId, request.citations);
        // Initiate validation pool
        uint poolIndex = dao.initiateValidationPool{value: request.fee}(
            request.evidenceContentId,
            POOL_DURATION,
            [uint256(1), uint256(3)],
            [uint256(1), uint256(2)],
            100,
            true,
            false,
            ""
        );
        // We have an approval from stake.worker to transfer up to stake.amount
        dao.delegatedStakeOnValidationPool(
            poolIndex,
            stake.worker,
            stake.amount,
            true
        );
    }
}
